/*
* media_player.c
*
* Media Player - Gsteramer based media player to display metrics data
*
* This is a media player that displays avg-fps, curr fps, drop rate, rendered and
* dropped frame data at specified interval which is configurable. It also display 
* width, height and Bitrate of file. Audio codec and video codec used for playback
* is also displayed. This complete metrics data is also sent to smart server for 
* tracing.
*
* Sample output:
*
* Bitrate of file is 1206992 
* video_codec is MPEG-4 video 
* audio_codec is MPEG-1 layer 3 
* width x height is 1280 x 720
*
* Pid=3819 Rend=41 Dropped=1 Width=320 Height=240 Curfps=26.52 dps=0.00 avgfps=41.00
* Pid=3819 Rend=53 Dropped=1 Width=320 Height=240 Curfps=23.46 dps=0.00 avgfps=26.50
* Pid=3819 Rend=66 Dropped=1 Width=320 Height=240 Curfps=26.52 dps=0.00 avgfps=33.00
*       
* Copyright (C) 2010

* Author(s):
*
*   Vishal Raj  <vishal.raj@linaro.org>
*               <vishal.raj@stericsson.com>
*
* Released under GPL v2
*/


#include <gst/gst.h>
#include <gst/video/video.h>
#include <string.h>
#include "media_player.h"

static GstElement *pipeline;
static GstElement *bin;
static GstElement *fpsdispsink;
static gpointer data;
static GstPad     *pad;
static GstStructure *gstr; 
static GstCaps *gcaps; 
static GstTagList *tag_list;
static char *video_codec,*audio_codec;
typedef struct { 
    pid_t	pid;
    guint       rate;	
    guint       width ;
    guint       height;	
    gdouble     curfps;
    gdouble     dps;
    gdouble     avgfps;
    unsigned long int     dropped;
    unsigned long int     rendered;
    char vcodec[64];
    char acodec[64];
}mm_data;

static mm_data mmdata;
static mm_data *mmdataptr;

static gboolean

fps_measurement_cb(GstElement *fpsdispsink,gdouble curfps,gdouble dps, gdouble avgfps)
{
	mmdataptr->curfps = curfps;
	mmdataptr->dps = dps;
	mmdataptr->avgfps = avgfps;

	g_object_get (G_OBJECT (fpsdispsink), "frames-rendered", &mmdataptr->rendered, NULL);
	g_object_get (G_OBJECT (fpsdispsink), "frames-dropped", &mmdataptr->dropped, NULL);

	if(!mmdataptr->width)
	{
   		pad = gst_element_get_pad(fpsdispsink,"sink");
 		if((gcaps = gst_pad_get_negotiated_caps (pad)) && (gstr = gst_caps_get_structure (gcaps, 0)))
		{  
  			gst_structure_get_int (gstr, "width", &mmdataptr->width); 
  			gst_structure_get_int (gstr, "height", &mmdataptr->height); 	
			fprintf(stderr," width x height is %d x %d\n ",mmdataptr->width,mmdataptr->height);
		}
		gst_caps_unref(gcaps);	
	}

    	fprintf(stderr,"Pid=%u Rend=%lu Dropped=%lu Width=%d Height=%d Curfps=%4.2f dps=%4.2f avgfps=%4.2f\n", 
                                                     mmdata.pid,
                                                     (unsigned long int)mmdata.rendered,
                                                     (unsigned long int)mmdata.dropped,     
                                                     mmdata.width,
                                                     mmdata.height,  
                                                     mmdata.curfps,   
                                                     mmdata.dps,      
                                                     mmdata.avgfps);   
    	player_send();
}

static gboolean
bus_cb (GstBus *bus,
        GstMessage *msg,
        gpointer data)
{	     
    switch (GST_MESSAGE_TYPE (msg))
    {
        case GST_MESSAGE_EOS:
            {
                g_debug ("end-of-stream");
    		player_stop ();
		exit(0);
                //break;
            }
	case GST_MESSAGE_TAG :
	    {

		gst_message_parse_tag (msg,&tag_list);
		if ( gst_tag_list_get_uint(tag_list,GST_TAG_BITRATE,&mmdataptr->rate))
			fprintf(stderr," Bitrate of file is %d \n",mmdataptr->rate);
		if ( gst_tag_list_get_string(tag_list, GST_TAG_VIDEO_CODEC,&video_codec))
		{			
			strcpy(mmdataptr->vcodec,video_codec);
			fprintf(stderr," video_codec is %s \n",mmdataptr->vcodec);
		}		
		if ( gst_tag_list_get_string(tag_list, GST_TAG_AUDIO_CODEC,&audio_codec))
		{
			strcpy(mmdataptr->acodec,audio_codec);			
			fprintf(stderr," audio_codec is %s \n",mmdataptr->acodec);
		}
		break;
	    }
        case GST_MESSAGE_ERROR:
            {
                gchar *debug;
                GError *err;

                gst_message_parse_error (msg, &err, &debug);
                g_free (debug);

                g_warning ("Error: %s", err->message);
                g_error_free (err);
		exit(0);
               // break;
            }
        default:
            break;
    }

    return TRUE;
}


void player_open(void)
{
    struct  hostent *hp;

    sock_desc = socket(AF_INET,SOCK_DGRAM,0);
    server.sin_family = AF_INET;
    server.sin_port  = (4200);

    hp = gethostbyname("127.0.0.1");
    bcopy (hp->h_addr,&(server.sin_addr),hp->h_length);
}

void
player_init (int *argc,
              char **argv[])
{
    mmdataptr = &mmdata;
    mmdataptr->pid = getpid();	
    gst_init (argc, argv);
    player_send();
}

void
player_play (const gchar *filename, const gint fps_update_interval)
{
    player_stop ();

    pipeline = gst_pipeline_new ("gst-player");

    bin = gst_element_factory_make ("playbin2", "bin");

    fpsdispsink = gst_element_factory_make ("fpsdisplaysink", "fpsdispsink");

    g_object_set (G_OBJECT (fpsdispsink), "signal-fps-measurements", TRUE,"text-overlay",TRUE,"fps-update-interval",fps_update_interval,"sync",TRUE, NULL);
    g_signal_connect (fpsdispsink, "fps-measurements", G_CALLBACK (fps_measurement_cb), (gpointer) data);
   
    g_object_set (G_OBJECT (bin), "video-sink", fpsdispsink, NULL);

    {
        GstBus *bus;
        bus = gst_pipeline_get_bus (GST_PIPELINE (pipeline));
        gst_bus_add_watch (bus, bus_cb, NULL);
        gst_object_unref (bus);
    }

    {
        gchar *uri;

        if (gst_uri_is_valid (filename))
        {
            uri = g_strdup (filename);
        }
        else if (g_path_is_absolute (filename))
        {
            uri = g_filename_to_uri (filename, NULL, NULL);
        }
        else
        {
            gchar *tmp;
            tmp = g_build_filename (g_get_current_dir (), filename, NULL);
            uri = g_filename_to_uri (tmp, NULL, NULL);
            g_free (tmp);
        }

        g_debug ("%s", uri);
        g_object_set (G_OBJECT (bin), "uri", uri, NULL);
        g_free (uri);
    }
   gst_bin_add (GST_BIN (pipeline), bin);
   gst_element_set_state (pipeline, GST_STATE_PLAYING);
}

void
player_send (void)
{
    sendto(sock_desc, (void *)&mmdata, sizeof(mmdata),0,(struct sockaddr *) &server, sizeof(server));
}

void
player_stop (void)
{
    if (pipeline)
    {   
        gst_element_set_state (pipeline, GST_STATE_NULL);
        gst_object_unref (GST_OBJECT (pipeline));
        pipeline = NULL;
    }
}


void
player_deinit (void)
{
}
