/*
* smartt-server.c
*
* System Metrics - Annotation Recording and Tracing Tool (SMARTT)  
*
* This is a client application that displays cpu-cycles, bus-cycles, instructions, 
* cache-misses, branches along with the player data such as FPS, Drop Rate etc
* The watemark levels for all the metrics can be set and are exclusively marked as
* watermark highligted regions.
*
* Sample output:
*
*[ PID(s) ][ TIME-STAMP ][ CPU-CYCLES (w) ][ CACHE-M'SS (w) ][ BRANCHES   (w) ]
*
* 3149      1298458764    5186403           3743842    (+)    1369802    (+)    
* 3149      1298458765    19817424   (+)    22701574          345118     (+)    
* 3149      1298458766    4727479           1447752    (+)    1168918        
*
* Copyright (C) Linaro 2010

* Author(s):
*
*   Sudip Jain  <sudip.jain@linaro.org>
*               <sudip.jain@stericsson.com>
*
* Released under GPL v2
*/

#include "smartt-network-services.c"
#include "smartt-server-utils.c"

#include "smartt-server.h"
#include "smartt-network-services.h"
#include "smartt-server-utils.h"


struct s_remote_client_handler remote_client_handler;
struct s_system_data system_data;
struct s_sys_info sys_info;
struct s_mem_info mem_info;
struct s_player_info player_info;

char *perf_args[64], *top_args[64];
char stat_counter_str[16], user_pid_str[16];
char pid_str[16], delay_str[16], iter_str[16];
unsigned int g_user_mode;

int main(void)
{
    SMARTTserver_Init();
    while (1)
    {                                                   /* repeat until done ....         */
        printf("\n[SMART-SERVER] # Listening... ");      /*   display a prompt             */
        fflush(stdout);
        recvfrom(smartt_client_rx_sock_desc, &remote_client_handler, sizeof(remote_client_handler), 0,(struct sockaddr *)&smartt_client_rx_server_addr, &addr_len);    
        if (strcmp(remote_client_handler.msg_id,"REQ:CONNECT") == 0)
            SMARTTserver_Start();
        else if (strcmp(remote_client_handler.msg_id,"REQ:RUN") == 0)
            SMARTTserver_Exec();
        else
            fprintf(stderr,"[E] Invalid Request\t\t\t: Failed\n");
    }
}

void SMARTTserver_Init()
{
    smartt_util_exec("clear");
    SMARTTserver_OpenSmarttClient_INPort();
    SMARTTserver_OpenSmarttPerf_INPort();
    SMARTTserver_OpenSmarttTop_INPort();
    SMARTTserver_OpenSmarttPlayer_INPort();

}

int SMARTTserver_Start()
{
    int status, n;

    printf("\n");
    printf("[I] SMART Server Started\t\t\t\t: Success\n");
    printf("[I] Waiting Request from SMART Client\t\t\t: Waiting\n");
    fflush(stdout);

    if ((status =  SMARTTserver_OpenSmarttClient_OUTPort(remote_client_handler.channel_config.host_ip, remote_client_handler.channel_config.port_no ))== 1)
    {
        sendto(smartt_client_tx_sock_desc, "ACK:ACCEPT", 64,0,(struct sockaddr *)&smartt_client_tx_server_addr, sizeof(smartt_client_tx_server_addr));
        fprintf(stderr,"[I] Connected to SMART Client @ %s:%d\t: Success\n",remote_client_handler.channel_config.host_ip,
                remote_client_handler.channel_config.port_no);
    } else
    {
        fprintf(stderr,"[I] Connected to SMART Client\t\t\t\t: Failed\n");
        return(0);
    }

    fprintf(stderr,"[I] Waiting for user configuration from SMART Client\t: Waiting\n");
    n = recvfrom(smartt_client_rx_sock_desc, &remote_client_handler, sizeof(remote_client_handler), 0,(struct sockaddr *)&smartt_client_rx_server_addr, &addr_len);    
    if (strcmp(remote_client_handler.msg_id,"REQ:USER-CONFIG") == 0)
    {
        fprintf(stderr,"[I] User Configuration Received\t\t\t\t: Success\n");
        sendto(smartt_client_tx_sock_desc, "ACK:USER-CONFIG",64,0,(struct sockaddr *) &smartt_client_tx_server_addr, sizeof(smartt_client_tx_server_addr));
    } else
    {
        fprintf(stderr,"[I] User Configuration Received\t\t\t\t: Failed\n");
        return(0);
    }

    if (remote_client_handler.pid == 0)
    {
        g_user_mode = 1;
        fprintf(stderr,"[I] Waiting for SMARTT Player to start \t\t\t: Waiting\n");
        n  = recvfrom(smartt_player_rx_sock_desc, &player_info, sizeof(player_info), 0, (struct sockaddr *)&smartt_player_rx_server_addr, &addr_len);    
        remote_client_handler.pid = player_info.pid;
        fprintf(stderr,"[I] Received SMART Player Process Id [%5u] \t\t: Success\n",player_info.pid);
    }
 
    n = recvfrom(smartt_client_rx_sock_desc, &remote_client_handler, sizeof(remote_client_handler), 0,(struct sockaddr *)&smartt_client_rx_server_addr, &addr_len);    
    if (strcmp(remote_client_handler.msg_id,"REQ:CAPTURE") == 0)
    {
        fprintf(stderr,"[I] Capturing System Metrics for PID [%5u] \t\t: Running\n",remote_client_handler.pid);
        SMARTTserver_Exec();

        fprintf(stderr,"[I] Capturing System Metrics for PID [%5u] \t\t: Completed\n",remote_client_handler.pid);
    } else
    {
        fprintf(stderr,"[I] Capturing System Metrics for PID [%5u] \t\t: Failed\n",remote_client_handler.pid);
        return(0);
    }
    return(1);
}

void SMARTTserver_FillPerfArgs()
{
    int i = 0;

    *(perf_args + i++) = "./perf";
    *(perf_args + i++) = "stat";
    *(perf_args + i++) = "-p";


    sprintf(user_pid_str,"%d",remote_client_handler.pid);
    *(perf_args + i++) = user_pid_str;

    if (remote_client_handler.event_config.event_enable[0] == 1)
    {
        *(perf_args + i++) = "-e";
        *(perf_args + i++) = "cpu-cycles";
    }
    if (remote_client_handler.event_config.event_enable[1] == 1)
    {
        *(perf_args + i++) = "-e";
        *(perf_args + i++) = "instructions";
    }
    if (remote_client_handler.event_config.event_enable[2] == 1)
    {
        *(perf_args + i++) = "-e";
        *(perf_args + i++) = "bus-cycles";
    }
    if (remote_client_handler.event_config.event_enable[3] == 1)
    {
        *(perf_args + i++) = "-e";
        *(perf_args + i++) = "cache-misses";
    }
    if (remote_client_handler.event_config.event_enable[4] == 1)
    {
        *(perf_args + i++) = "-e";
        *(perf_args + i++) = "branches";
    }
    *(perf_args + i++) = "sleep";
    sprintf(stat_counter_str,"%4.2lf",remote_client_handler.event_config.counter);
    *(perf_args + i++) = stat_counter_str;
    *(perf_args + i)   = '\0';
}

void SMARTTserver_FillTopArgs()
{
    int i = 0;

    *(top_args + i++) = "./top";

    *(top_args + i++) = "-p";
//    sprintf(pid_str,"%u",9478);
    sprintf(pid_str,"%d",remote_client_handler.pid);
    *(top_args + i++) = pid_str;

    *(top_args + i++) = "-d";
    sprintf(delay_str,"%4.2lf",remote_client_handler.event_config.counter);
//    sprintf(delay_str,"%4.2lf",1.0);
    *(top_args + i++) = delay_str;

    *(top_args + i++) = "-n";
    sprintf(iter_str,"%u",1);
    *(top_args + i++) = iter_str;

    *(top_args + i)   = '\0';
}

void SMARTTserver_Exec()
{
    int i;

    SMARTTserver_FillPerfArgs();
    SMARTTserver_FillTopArgs();
    for (i=1; i<=remote_client_handler.event_config.samples; i++)
    {
        smartt_util_execute(perf_args);
        smartt_util_execute(top_args);

        SMARTTserver_Capture();
        SMARTTserver_Process();
     }
}

void SMARTTserver_Capture()
{
    unsigned int n;

    n  = recvfrom(smartt_perf_rx_sock_desc, &sys_info, sizeof(sys_info), 0, (struct sockaddr *)&smartt_perf_rx_server_addr, &addr_len);    
    n  = recvfrom(smartt_top_rx_sock_desc, &mem_info, sizeof(mem_info), 0, (struct sockaddr *)&smartt_top_rx_server_addr, &addr_len);
    if (g_user_mode)
        n  = recvfrom(smartt_player_rx_sock_desc, &player_info, sizeof(player_info), 0, (struct sockaddr *)&smartt_player_rx_server_addr, &addr_len);    
}

void SMARTTserver_Process()
{
    system_data.sys_info    = sys_info;
    system_data.mem_info    = mem_info;
    system_data.player_info = player_info;

    sendto(smartt_client_tx_sock_desc, (void *)&system_data, sizeof(system_data),0,(struct sockaddr *) &smartt_client_tx_server_addr, sizeof(smartt_client_tx_server_addr));
    printf("[R] Sys info captured and send to server at epoch time [%lu] \n", system_data.sys_info.time_stamp);
    fflush(stdout);
}

void SMARTTserver_Exit()
{
    printf("[I] Exitting SMARTT Server\t\t\t\t: Success\n");
    exit(0);
}

void SMARTTserver_Test()
{
    int i=0;
    SMARTTserver_FillTopArgs();
    smartt_util_execute(top_args);
    while (getchar()!='\0')
        printf("%s",*(top_args + i++));
}


